#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

from M2Crypto import X509
import lib.miarc_util

class PlantillaUso:
	""" 
	Define las plantillas de uso por defecto que incorpora miARC para los 
	certificados y se corresponden con los diez usos definidos en la 
	documentación de OpenSSL.

	http://www.openssl.org/docs/apps/x509.html
	http://www.ietf.org/rfc/rfc3280.txt
	"""

	def __init__(self, uso, certificado):
		if uso == 1:										# SSL Client
			ext1 = ExtendedKeyUsage(['clientAuth'], False)
			ext2 = KeyUsage(['digitalSignature'], False)
			ext3 = NetscapeCertificateType (['client'])
			ext4 = NetscapeStringExtension(1, "miARC SSL Client Certificate")
		elif uso == 2:										# SSL Client CA
			ext1 = BasicConstraint(True, -1, True)
			ext2 = KeyUsage(['keyCertSign'], False)
			ext3 = ExtendedKeyUsage(['clientAuth'], False)
			ext4 = NetscapeCertificateType (['sslCA'])
			ext5 = NetscapeStringExtension(1, "miARC SSL Client CA Certificate")
		elif uso == 3:										# SSL Server
			ext1 = ExtendedKeyUsage(['clientAuth', 'msSGC'], False)
			ext2 = KeyUsage(['digitalSignature', 'keyEncipherment'], False)
			ext3 = NetscapeCertificateType (['server'])
			ext4 = NetscapeStringExtension(1, "miARC SSL Server Certificate")
		elif uso == 4:										# SSL Server CA
			ext1 = BasicConstraint(True, -1, True)
			ext2 = KeyUsage(['keyCertSign'], False)
			ext3 = ExtendedKeyUsage(['clientAuth', 'msSGC'], False)
			ext4 = NetscapeCertificateType (['sslCA'])
			ext5 = NetscapeStringExtension(1, "miARC SSL Server CA Certificate")
		elif uso == 5:										# Netscape SSL Server
			ext1 = KeyUsage(['keyEncipherment'], False)
			ext2 = ExtendedKeyUsage(['clientAuth', 'nsSGC'], False)
			ext3 = NetscapeCertificateType (['server'])
			ext4 = NetscapeStringExtension(1, "miARC Netscape SSL Server Certificate")
		elif uso == 6:										# S/MIME Signing
			ext1 = ExtendedKeyUsage(['emailProtection'], False)
			ext2 = NetscapeCertificateType (['email'])
			ext3 = KeyUsage(['digitalSignature'], False)
			ext4 = NetscapeStringExtension(1, "miARC S/MIME Signing Certificate")
		elif uso == 7:										# S/MIME Encryption
			ext1 = ExtendedKeyUsage(['emailProtection'], False)
			ext2 = NetscapeCertificateType (['email'])
			ext3 = KeyUsage(['keyEncipherment'], False)
			ext4 = NetscapeStringExtension(1, "miARC S/MIME Encryption Certificate")
		elif uso == 8:										# S/MIME CA
			ext1 = BasicConstraint(True, -1, True)
			ext2 = KeyUsage(['keyCertSign'], False)
			ext3 = ExtendedKeyUsage(['emailProtection'], False)
			ext4 = NetscapeCertificateType (['emailCA'])
			ext5 = NetscapeStringExtension(1, "miARC S/MIME CA Certificate")
		elif uso == 9:										# CRL Signing
			ext1 = KeyUsage(['cRLSign'], False)
			ext2 = NetscapeStringExtension(1, "miARC CRL Signing Certificate")
		elif uso == 10:										# CRL Signing CA
			ext1 = BasicConstraint(True, -1, True)
			ext2 = KeyUsage(['cRLSign', 'keyCertSign'], False)
			ext3 = NetscapeStringExtension(1, "miARC CRL Signing CA Certificate")

		certificado.add_ext(ext1.ext)
		certificado.add_ext(ext2.ext)
		if uso == 10:
			certificate.add_ext(ext3.ext)		
		elif uso == 1 or uso == 3 or uso == 5 or uso == 6 or uso == 7:
			certificado.add_ext(ext3.ext)		
			certificado.add_ext(ext4.ext)
		else:
			certificado.add_ext(ext3.ext)		
			certificado.add_ext(ext4.ext)
			certificado.add_ext(ext5.ext)


# Extensiones Estandar
# X509.new_extension(name, value, critical=0, _pyfree=1):

class BasicConstraint:
	"""
	This is a multi valued extension which indicates whether a certificate is a CA certificate.
	"""
	def __init__(self, ca, pathlen, critica):
		""" 
		Método constructor.

		p1 ca: tipo boolean indicando si el certificado puede actuar como certificado de CA.
		p2 pathlen: entero no negativo, en el caso de que actue como certificado raiz de CA la longitud de la cadena de certs hijos que permite.
					* pathlen < 0: significa pathlen infinito.
		p3 critica: tipo boolean indicando si la extensión es crítica.
		"""

		if pathlen < 0:
			self.ext = X509.new_extension('basicConstraints', "CA:"+str(ca).lower(), critical=int(critica))
		else:
			self.ext = X509.new_extension('basicConstraints', "CA:"+str(ca).lower()+", pathlen:"+str(pathlen), critical=int(critica))


class KeyUsage:
	"""
	Key usage is a multi valued extension consisting of a list of names of the permitted key usages.
	"""

	def __init__(self, usosPermitidos, critica):
		""" 
		Método constructor.

		p1 usosPermitidos: lista de cadenas con los usos permitidos a la clave, los posibles valores son:
		   digitalSignature, nonRepudiation, keyEncipherment, dataEncipherment, keyAgreement, keyCertSign, cRLSign, encipherOnly y decipherOnly.
		p2 critica: tipo boolean indicando si la extensión es crítica
		"""

		self.ext = X509.new_extension('keyUsage', lib.miarc_util.listaToCadena(usosPermitidos), critical=int(critica))


class ExtendedKeyUsage:
	"""
	This extensions consists of a list of usages indicating purposes for which the certificate public key can be used for.
	"""
	
	def __init__(self, usosPermitidos, critica):
		""" 
		Método constructor.

		p1 usosPermitidos: lista de cadenas con los usos permitidos a la clave, los posibles valores son:
			 Value                  Meaning
			 -----                  -------
			 serverAuth             SSL/TLS Web Server Authentication.
			 clientAuth             SSL/TLS Web Client Authentication.
			 codeSigning            Code signing.
			 emailProtection        E-mail Protection (S/MIME).
			 timeStamping           Trusted Timestamping
			 msCodeInd              Microsoft Individual Code Signing (authenticode)
			 msCodeCom              Microsoft Commercial Code Signing (authenticode)
			 msCTLSign              Microsoft Trust List Signing
			 msSGC                  Microsoft Server Gated Crypto
			 msEFS                  Microsoft Encrypted File System
			 nsSGC                  Netscape Server Gated Crypto
		p2 critica: tipo boolean indicando si la extensión es crítica
		"""

		self.ext = X509.new_extension('extendedKeyUsage', lib.miarc_util.listaToCadena(usosPermitidos), critical=int(critica))


class SubjectKeyIdentifier:
	""" 
	This is really a string extension and can take two possible values. 

	Either the word hash which will automatically follow the guidelines in RFC3280 or a hex string giving the extension value to include. The use of the hex string is strongly discouraged. 
	*IMPORTANTE: m2Crypto no soporta el argumento hash, value must be precomputed hash.
	"""

	def __init__(self, hashPrecomputado, critica):
		""" 
		Método constructor.
		"""
		self.ext = X509.new_extension('subjectKeyIdentifier', hashPrecomputado, critical=int(critica))


class AuthorityKeyIdentifier:
	"""
	The authority key identifier extension permits two options. keyid and issuer: both can take the optional value "always".

	If the keyid option is present an attempt is made to copy the subject key identifier from the parent certificate. 
	If the value "always" is present then an error is returned if the option fails.
	The issuer option copies the issuer and serial number from the issuer certificate. This will only be done if the keyid option fails or is not included unless the "always" flag will always include the value.
	"""
# TODO testear en un certificado
	def __init__(self, opciones, critica):
#		self.ext = X509.new_extension('authorityKeyIdentifier', diccionarioToCadena(opciones), critical=int(critica))
		self.ext = X509.new_extension('authorityKeyIdentifier', opciones, critical=int(critica))
# keyid:always, issuer:always


class SubjectAlternativeName:
	"""
	The subject alternative name extension allows various literal values to be included in the configuration file.

	These include email (an email address) URI a uniform resource indicator, DNS (a DNS domain name), RID (a registered ID: OBJECT IDENTIFIER), IP (an IP address), dirName (a distinguished name) and otherName.
	"""
# TODO testear la opción email:copy en un certificado
# si no funciona lo haremos copiando el email del subject name manualmente
	def __init__(self, critica):
#		self.ext = X509.new_extension('subjectAltName', "email:copy,email:my@other.address,URI:http://my.url.here/", critical=int(critica))
		self.ext = X509.new_extension('subjectAltName', "email:jnsad@asdas.com, email:my@other.address, URI:http://my.url.here/, DNS:dominio.com, RID:1.2.3.4, IP:192.168.7.1, otherName:1.2.3.8;UTF8:some other identifier", critical=int(critica))


class IssuerAlternativeName:
	"""
	The issuer alternative name option supports all the literal options of subject alternative name. 

	It does not support the email:copy option because that would not make sense. 
	It does support an additional issuer:copy option that will copy all the subject alternative name values from the issuer certificate (if possible). 
	"""
	def __init__(self, critica):
		self.ext = X509.new_extension('issuerAltName', "email:jnsad@asdas.com, email:my@other.address, URI:http://my.url.here/, DNS:dominio.com, RID:1.2.3.4, IP:192.168.7.1, otherName:1.2.3.8;UTF8:some other identifier", critical=int(critica))


class  AuthorityInfoAccess:
	"""
	The authority information access extension gives details about how to access certain information relating to the CA. 
	
	Its syntax is accessOID;location where location has the same syntax as subject alternative name (except that email:copy is not supported). accessOID can be any valid OID but only certain values are meaningful, for example OCSP and caIssuers. 
	"""
	def __init__(self, critica):
		self.ext = X509.new_extension('authorityInfoAccess', "caIssuers;URI:http://my.ca/ca.html", critical=int(critica))
# tb	self.ext = X509.new_extension('authorityInfoAccess', "caIssuers;URI:http://my.ca/ca.html", critical=int(critica))


def listaURLsToCadena (lista):
	cadena = "URI:http://"
	x = 0
	while x<(len(lista)-1):
		cadena = cadena + lista[x] + ', URI:http://'
		x = x+1
	cadena = cadena + lista[len(lista)-1]
	return cadena


class CRLDistributionPoints:
	"urls es una lista de direcciones donde se encuentra la crl sin incluir el http://"
	def __init__(self, urls, critica):
		self.ext = X509.new_extension('crlDistributionPoints', listaURLsToCadena(urls), critical=int(critica))

class IssuingDistributionPoint:
	def __init__(self, critica):
		pass

def diccionarioToCadenaPC (diccionario):
	cadena = str(diccionario)
	cadena = cadena[1:len(cadena)-1]
	cadena = cadena.replace("""'""",'')
	return cadena

class PolicyConstraints:
	"This is a multi-valued extension which consisting of the names requireExplicitPolicy or inhibitPolicyMapping and a non negative intger value. At least one component must be present."
	def __init__(self, valores, critica):
		self.ext = X509.new_extension('policyConstraints', diccionarioToCadenaPC(valores), critical=int(critica))
#		self.ext = X509.new_extension('policyConstraints', 'requireExplicitPolicy:3', critical=int(critica))
#		self.ext = X509.new_extension('policyConstraints', 'inhibitPolicyMapping:3' , critical=int(critica))
#		self.ext = X509.new_extension('policyConstraints', 'requireExplicitPolicy:3, inhibitPolicyMapping:5' , critical=int(critica))

class InhibitAnyPolicy:
	"""
	This is a string extension whose value must be a non negative integer.
	"""
	def __init__(self, numero, critica):
		if numero>=0:
			self.ext = X509.new_extension('inhibitAnyPolicy', str(numero), critical=int(critica))
		else:
			pass # ERROR


class NameConstraints:
	"""
	The name constraints extension is a multi-valued extension. The name should begin with the word permitted or excluded followed by a ;. The rest of the name and the value follows the syntax of subjectAltName except email:copy is not supported and the IP form should consist of an IP addresses and subnet mask separated by a /. 
	"""
	def __init__(self, opciones, critica):
		self.ext = X509.new_extension('nameConstraints', "permitted;IP:192.168.0.0/255.255.0.0, permitted;email:holap@somedomain.com, excluded;email:.com, excluded;URI:http://my.url.here/, excluded;DNS:dominio.com, excluded;RID:1.2.3.4", critical=int(critica))
#nameConstraints


class OCSPNoCheck:
	"""
	The OCSP No Check extension is a string extension but its value is ignored.
	"""
	def __init__(self, cadena, critica):
		"""
		p1 cadena: nombre de una extensión.
		"""
		self.ext = X509.new_extension('noCheck', cadena, critical=int(critica))



#Extensiones Depreciadas
class NetscapeStringExtension:
	"""Netscape Comment (nsComment) is a string extension containing a comment which will be displayed when the certificate is viewed in some browsers.
		Example: nsComment = "Some Random Comment"
		Other supported extensions in this category are: nsBaseUrl, nsRevocationUrl, nsCaRevocationUrl, nsRenewalUrl, nsCaPolicyUrl and nsSslServerName. 
		nsComment = 1, nsBaseUrl = 2, nsRevocationUrl = 3, nsCaRevocationUrl = 4, nsRenewalUrl = 5, nsCaPolicyUrl = 6 and nsSslServerName = 7"""
	def __init__(self, subcategoria, valor):
		if subcategoria == 1:
			self.ext = X509.new_extension('nsComment', valor)
		elif subcategoria == 2:
			self.ext = X509.new_extension('nsBaseUrl', valor)
		elif subcategoria == 3:
			self.ext = X509.new_extension('nsRevocationUrl', valor)
		elif subcategoria == 4:
			self.ext = X509.new_extension('nsCaRevocationUrl', valor)
		elif subcategoria == 5:
			self.ext = X509.new_extension('nsRenewalUrl', valor)
		elif subcategoria == 6:
			self.ext = X509.new_extension('nsCaPolicyUrl', valor)
		elif subcategoria == 7:
			self.ext = X509.new_extension('nsSslServerName', valor)


class NetscapeCertificateType:
	"""This is a multi-valued extensions which consists of a list of flags to be included. It was used to indicate the purposes for which a certificate could be used. The basicConstraints, keyUsage and extended key usage extensions are now used instead.
Acceptable values for nsCertType are: client, server, email, objsign, reserved, sslCA, emailCA, objCA. """
	def __init__(self, listaTipos):
		self.ext = X509.new_extension('nsCertType', lib.miarc_util.listaToCadena(listaTipos))
